/*
Gwenview - A simple image viewer for KDE
Copyright (C) 2000-2001 Aurlien Gteau

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

// QT includes
#include <qpixmap.h>

// KDE includes
#include <kapp.h>
#include <kdebug.h>
#include <kfiledialog.h>
#include <kglobal.h>
#include <kio/job.h>
#include <klineeditdlg.h>
#include <klocale.h>
#include <kmessagebox.h>

// Our includes
#include "filednddetailview.h"
#include "fileoperation.h"
#include "filethumbnailview.h"
#include "imagefilereader.h"

#include "filewidget.moc"

// -------------------------------------------------------------
static const char* CONFIG_CONFIRM_DELETE="confirm file delete";
static const char* CONFIG_CONFIRM_MOVE="confirm file move";
static const char* CONFIG_CONFIRM_COPY="confirm file copy";
static const char* CONFIG_MOVE_FOLDER="default move folder";
static const char* CONFIG_COPY_FOLDER="default copy folder";


// -------------------------------------------------------------
FileWidget::FileWidget(QWidget* parent,Mode mode)
: QWidgetStack(parent)
{
// File reader
	mFileReader=new ImageFileReader();

	connect(mFileReader,SIGNAL(newItems(const KFileItemList&)),
		this,SLOT(newItems(const KFileItemList&)) );
	connect(mFileReader,SIGNAL(clear()),
		this,SLOT(clear()) );
	connect(mFileReader,SIGNAL(deleteItem(KFileItem*)),
		this,SLOT(removeItem(KFileItem*)) );
	connect(mFileReader,SIGNAL(completed()),
		this,SLOT(fileReaderCompleted()) );
	connect(mFileReader,SIGNAL(canceled()),
		this,SLOT(fileReaderCanceled()) );

// Propagate completed and canceled signals
	connect(mFileReader,SIGNAL(completed()),
		this,SIGNAL(completed()) );
	connect(mFileReader,SIGNAL(canceled()),
		this,SIGNAL(canceled()) );

// File detail widget
	mFileDetailView=new FileDnDDetailView(this,"filedetailview");
	addWidget(mFileDetailView,0);

	connect(mFileDetailView,SIGNAL(clicked(QListViewItem*)),this,SLOT(detailChanged(QListViewItem*)) );
	connect(mFileDetailView,SIGNAL(returnPressed(QListViewItem*)),this,SLOT(detailChanged(QListViewItem*)) );

// Thumbnail widget
	mFileThumbnailView=new FileThumbnailView(this);
	addWidget(mFileThumbnailView,1);

	connect(mFileThumbnailView,SIGNAL(clicked(QIconViewItem*)),this,SLOT(thumbnailChanged(QIconViewItem*)) );
	connect(mFileThumbnailView,SIGNAL(returnPressed(QIconViewItem*)),this,SLOT(thumbnailChanged(QIconViewItem*)) );

	setMode(mode);
}


FileWidget::~FileWidget() {
	delete mFileReader;
}


KFileView* FileWidget::currentFileView() const {
	if (mMode==FileList) {
		return mFileDetailView;
	} else {
		return mFileThumbnailView;
	}
}


//-Public slots----------------------------------------------------------
void FileWidget::setFolderAndFile(QString folder,QString filename) {
	if (mFilename==filename && mFolder==folder) {
		return;
	}

	mFilename=filename;
	mFolder=folder;
	mFileThumbnailView->stopThumbnailUpdate();

	mFileReader->openURL(KURL(mFolder),true);
}


void FileWidget::cancel() {
	if (!mFileReader->isFinished()) {
		mFileReader->stop();
		return;
	}
	if (mMode==Thumbnail) {
		mFileThumbnailView->stopThumbnailUpdate();
	}
}


void FileWidget::selectFilename(QString filename) {
	if (mFilename==filename) return;

	selectFilenameInternal();

	emit filenameChanged(mFilename);
}


void FileWidget::selectFilenameInternal() {
	KFileViewItem* item;
	currentFileView()->setCurrentItem(mFilename);
	item=currentFileView()->selectedItems()->getFirst();
	if (item) {
		currentFileView()->ensureItemVisible(item);
	}
}


void FileWidget::selectFirst() {
	KFileViewItemListIterator it(* (currentFileView()->items()) );
	KFileViewItem* item=it.toFirst();
	if (!item)
		return;

	currentFileView()->setSelected(item,true);
	currentFileView()->ensureItemVisible(item);
	mFilename=item->name();
	emit filenameChanged(mFilename);
}


void FileWidget::selectLast() {
	KFileViewItemListIterator it(* (currentFileView()->items()) );
	KFileViewItem* item=it.toLast();
	if (!item)
		return;

	currentFileView()->setSelected(item,true);
	currentFileView()->ensureItemVisible(item);
	mFilename=item->name();
	emit filenameChanged(mFilename);
}


void FileWidget::selectPrevious() {

	KFileViewItemListIterator it(* (currentFileView()->items()) );

// Look for the current item
	for(it.toLast();!it.atFirst();--it) {
	// We found it, select previous one
		if ( currentFileView()->isSelected(*it) ) {
			--it;
			currentFileView()->setSelected(*it,true);
			currentFileView()->ensureItemVisible(*it);
			mFilename=(*it)->name();
			emit filenameChanged(mFilename);
			return;
		}
	}
}


void FileWidget::selectNext() {
	KFileViewItemListIterator it(* (currentFileView()->items()) );

// Look for the current item
	for(;!it.atLast();++it) {
	// We found it, select next one
		if ( currentFileView()->isSelected(*it) ) {
			++it;
			currentFileView()->setSelected(*it,true);
			currentFileView()->ensureItemVisible(*it);
			mFilename=(*it)->name();
			emit filenameChanged(mFilename);
			return;
		}
	}
}


void FileWidget::detailChanged(QListViewItem* item) {
	if (item) {
		mFilename=item->text(0);
	} else {
		mFilename="";
	}
	emit filenameChanged(mFilename);
}


void FileWidget::thumbnailChanged(QIconViewItem* item) {
	if (item) {
		mFilename=item->text();
	} else {
		mFilename="";
	}
	emit filenameChanged(mFilename);
}


void FileWidget::setMode(FileWidget::Mode mode) {
	mMode=mode;

	if (mMode==FileList) {
		mFileThumbnailView->stopThumbnailUpdate();
		raiseWidget(mFileDetailView);
	} else {
		raiseWidget(mFileThumbnailView);
	}

	mFileReader->listDirectory();
}


void FileWidget::setConfirmDelete(bool value) {
	mConfirmDelete=value;
}


void FileWidget::setConfirmCopy(bool value) {
	mConfirmCopy=value;
}


void FileWidget::setConfirmMove(bool value) {
	mConfirmMove=value;
}


void FileWidget::setCopyToFolder(const QString& url) {
	mCopyToFolder=url;
}


void FileWidget::setMoveToFolder(const QString& url) {
	mMoveToFolder=url;
}


//-Accessors-------------------------------------------------------------
bool FileWidget::currentIsFirst() const {
	KFileViewItem* currentItem=currentFileView()->selectedItems()->getFirst();
	if (!currentItem) return false;

	KFileViewItemListIterator it(* (currentFileView()->items()) );
	KFileViewItem* firstItem=it.toFirst();
	if (!firstItem) return false;

	return currentItem==firstItem;
}


bool FileWidget::currentIsLast() const {
	KFileViewItem* currentItem=currentFileView()->selectedItems()->getFirst();
	if (!currentItem) return false;

	KFileViewItemListIterator it(* (currentFileView()->items()) );
	KFileViewItem* lastItem=it.toLast();
	if (!lastItem) return false;

	return currentItem==lastItem;
}


uint FileWidget::fileCount() const {
	return currentFileView()->items()->count();
}


//-File reader slots-----------------------------------------------------
void FileWidget::removeItem(KFileItem* item) {
	currentFileView()->removeItem((KFileViewItem*)item);
}


void FileWidget::newItems(const KFileItemList& items) {
	currentFileView()->addItemList(items);
}


void FileWidget::clear() {
	currentFileView()->clear();
}


void FileWidget::fileReaderCompleted() {
	if (mFilename.isEmpty()) {
		selectFirst();
	} else {
		selectFilenameInternal();
	}

	if (mMode==Thumbnail) {
		mFileThumbnailView->startThumbnailUpdate();
	}
}


void FileWidget::fileReaderCanceled() {
	if (mMode==Thumbnail) {
		mFileThumbnailView->stopThumbnailUpdate();
	}
	if (mFilename.isEmpty()) {
		selectFirst();
	} else {
		selectFilenameInternal();
	}
}


void FileWidget::stopFileReader() {
	mFileReader->stop();
}


//-File operations-------------------------------------------------------
void FileWidget::deleteFile() {
	if (mConfirmDelete) {
// Ask confirmation
		int response=KMessageBox::questionYesNo(this,
			i18n("Are you sure you want to delete file <b>%1</b> ?").arg(mFilename));
		if (response==KMessageBox::No) return;
	}

// Get the current file, then select the next one
// (or the previous one if we want to delete the last item)
	QString fullPath=mFolder + '/' + mFilename;
	if (currentIsLast()) {
		selectPrevious();
	} else {
		selectNext();
	}

// Delete the file
	KURL url;
	url.setPath(fullPath);
	FileOperation::fileDelete(url,this);
}



void FileWidget::renameFile() {
	bool ok;

// Prompt for the new filename
	mNewFilename=KLineEditDlg::getText(
		i18n("Rename file <b>%1</b> to :").arg(mFilename),
		mFilename,
		&ok,this);

	if (!ok) return;

// Rename the file
	KURL src,dst;
	src.setPath(mFolder + '/' + mFilename);
	dst.setPath(mFolder + '/' + mNewFilename);
	KIO::Job* job=KIO::file_move(src,dst);
	connect( job, SIGNAL( result(KIO::Job*) ),
		this, SLOT( slotRenamed(KIO::Job*) ) );
}


void FileWidget::slotRenamed(KIO::Job* job) {
// Show error if any
	if (job->error()) {
		job->showErrorDialog(this);
		return;
	}

// Refresh the view and takes to select the renamed item
	mFileReader->setURLDirty(true);
	mFilename=mNewFilename;
	mFileReader->listDirectory();
}


void FileWidget::copyFile() {
	QString newFolder;

	if (mConfirmCopy) {
	// Prompt for the new folder
		newFolder=KFileDialog::getExistingDirectory(mCopyToFolder, this);
	} else {
		newFolder=mCopyToFolder;
	}
	if (newFolder.isEmpty()) return;

// Copy the file
	KURL src,dst;
	src.setPath(mFolder + '/' + mFilename);
	dst.setPath(newFolder + '/' + mFilename);
	FileOperation::fileCopy(src,dst,this);
}


void FileWidget::moveFile() {
	QString dstPath;

	if (mConfirmMove) {
	// Prompt for the new folder
		dstPath=KFileDialog::getExistingDirectory(mMoveToFolder, this);
	} else {
		dstPath=mMoveToFolder;
	}
	if (dstPath.isEmpty()) return;
	dstPath+='/' + mFilename;

// Get the current file, then select the next one
// (or the previous one if we want to delete the last item)
	QString srcPath=mFolder + '/' + mFilename;
	if (currentIsLast()) {
		selectPrevious();
	} else {
		selectNext();
	}

// Move the file
	KURL src,dst;
	src.setPath(srcPath);
	dst.setPath(dstPath);
	FileOperation::fileMove(src,dst,this);
}


//-Configurable implementation----------------------------------------------
void FileWidget::readConfig(KConfig* config,const QString& group) {
	mFileThumbnailView->readConfig(config,group);

	config->setGroup(group);
	mConfirmDelete=config->readBoolEntry(CONFIG_CONFIRM_DELETE,true);
	mConfirmMove=config->readBoolEntry(CONFIG_CONFIRM_MOVE,true);
	mConfirmCopy=config->readBoolEntry(CONFIG_CONFIRM_COPY,true);

	mMoveToFolder=config->readEntry(CONFIG_MOVE_FOLDER);
	mCopyToFolder=config->readEntry(CONFIG_COPY_FOLDER);
}


void FileWidget::writeConfig(KConfig* config,const QString& group) const {
	mFileThumbnailView->writeConfig(config,group);

	config->setGroup(group);
	config->writeEntry(CONFIG_CONFIRM_DELETE,mConfirmDelete);
	config->writeEntry(CONFIG_CONFIRM_MOVE,mConfirmMove);
	config->writeEntry(CONFIG_CONFIRM_COPY,mConfirmCopy);

	config->writeEntry(CONFIG_MOVE_FOLDER,mMoveToFolder);
	config->writeEntry(CONFIG_COPY_FOLDER,mCopyToFolder);
}
