/*
Gwenview - A simple image viewer for KDE
Copyright (C) 2000-2001 Aurlien Gteau

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// QT includes
#include <qdir.h>
#include <qheader.h>
#include <qpopupmenu.h>
#include <qtimer.h>

// KDE includes
#include <kdebug.h>
#include <kdirlister.h>
#include <kiconloader.h>
#include <kio/global.h>
#include <kio/job.h>
#include <klocale.h>
#include <klineeditdlg.h>
#include <kmessagebox.h>
#include <kpropsdlg.h>

// Our includes
#include "folderwidget.moc"

#include "folderwidgetitem.h"



/**
 * Constructor
 * Create the root node and the popup menu
 */
FolderWidget::FolderWidget(QWidget* parent) : QListView(parent) {
	FolderWidgetItem* item;
	
	mPopupMenu=new QPopupMenu(this);
	mPopupMenu->insertItem(SmallIcon("folder_new"),i18n("New folder"),this,SLOT(makeDir()));
	mPopupMenu->insertSeparator();
	mPopupMenu->insertItem(i18n("Rename..."),this,SLOT(renameDir()));
	mPopupMenu->insertItem(SmallIcon("editdelete"),i18n("Delete"),this,SLOT(removeDir()));
	mPopupMenu->insertSeparator();
	mPopupMenu->insertItem(i18n("Properties"),this,SLOT(showPropertiesDialog()));

	mDirLister=new KDirLister;
	mDirLister->setDirOnlyMode(true);

	connect(this,SIGNAL(selectionChanged(QListViewItem*)),
		this,SLOT(onSelectionChanged(QListViewItem*)));
	
	connect(mDirLister, SIGNAL(newItems(const KFileItemList&)),
		this,SLOT(onNewItems(const KFileItemList&)));
	connect(mDirLister, SIGNAL(deleteItem(KFileItem*)),
		this,SLOT(onDeleteItem(KFileItem*)));
	connect(mDirLister, SIGNAL(refreshItems(const KFileItemList&)),
		this,SLOT(onRefreshItems(const KFileItemList&)));

	mNewDirTimer=new QTimer(this);

	connect(mNewDirTimer, SIGNAL(timeout()),
		this,SLOT(processNewDirQueue()));

	connect(this,SIGNAL(rightButtonClicked(QListViewItem*,const QPoint&,int)),
		this,SLOT(onRightButtonClicked(QListViewItem*,const QPoint&,int)));

	addColumn(i18n("Folders"));
	header()->hide();

	KURL root;
	root.setPath("/");
	item=new FolderWidgetItem(this,"/",root);
	item->setOpen(true);
}


FolderWidget::~FolderWidget() {
	delete mDirLister;
}


/**
 * Emit our folderChanged signal
 */
void FolderWidget::onSelectionChanged(QListViewItem* paramItem) {
	FolderWidgetItem* item=static_cast<FolderWidgetItem*>(paramItem);

	mURL.setPath(item->folder());
	emit folderChanged(item->folder());
}


/**
 * Init mURL and start the dir lister on it
 */
void FolderWidget::setFolder(QString newFolder) {

	KURL url;
	url.setPath(newFolder);

	if (mURL.cmp(url,true)) {
		return;
	}
	mURL=url;


	QStringList folderParts;
	QStringList::Iterator folderIter,endFolderIter;
	QString folder="/";
	FolderWidgetItem* viewItem;
	FolderWidgetItem* nextViewItem;

	folderParts=QStringList::split('/',newFolder);
	folderIter=folderParts.begin();
	endFolderIter=folderParts.end();
	viewItem=static_cast<FolderWidgetItem*>(firstChild());

// Finds the deepest existing view item
	for(;folderIter!=endFolderIter;++folderIter) {

		nextViewItem=viewItem->find(*folderIter);
		if (nextViewItem) {
			folder+=*folderIter + "/";
			viewItem=nextViewItem;
		} else {
			break;
		}
	}

	url.setPath(folder);
	mDirLister->openURL(url,true,true);

}


//-Dir lister slots-------------------------------------------------------
/**
 * Filters hidden items and calls addDir for each dir in the list
 */
void FolderWidget::onNewItems(const KFileItemList& items) {
	KFileItem* item;
	QListIterator<KFileItem> it(items);
	for(;it.current();++it) {
		item=*it;
		if (item->name()[0]!='.') {
			mNewDirQueue.append(item);
		}
	}
	if (!mNewDirTimer->isActive() && !mNewDirQueue.isEmpty()) {
		mNewDirTimer->start(0,false);
	}
}


void FolderWidget::onDeleteItem(KFileItem* fileItem) {
	if (mItemMap.contains(fileItem)) {
		FolderWidgetItem* viewItem=mItemMap[fileItem];
		delete viewItem;
		mItemMap.remove(fileItem);
    } else {
		kdWarning() << "  not found, shouldn't happen\n";
		return;
	}

	mNewDirQueue.removeRef(fileItem);

}



void  FolderWidget::onRefreshItems(const KFileItemList& items) {
	KFileItem* fileItem;
	FolderWidgetItem* viewItem;
	QListIterator<KFileItem> it(items);
	for(;it.current();++it) {
		fileItem=*it;

		viewItem=mItemMap[fileItem];
		if (!viewItem) {
			continue;
		}
		viewItem->setURL(fileItem->url());
		if (viewItem==currentItem()) {
			emit folderChanged(viewItem->folder());
		}
	}
}


//-Private-----------------------------------------------------------------
void FolderWidget::processNewDirQueue() {
	if (mNewDirQueue.isEmpty()) {
		mNewDirTimer->stop();
		return;
	}
	KFileItem* item=mNewDirQueue.getFirst();
	mNewDirQueue.removeFirst();
	addDir(item);
}


/**
 * Adds a new dir to the tree if it does not already exist
 */
void FolderWidget::addDir(KFileItem* fileItem) {
	QStringList folderParts;
	QStringList::Iterator folderIter,endFolderIter;
	QString folder="/";
	FolderWidgetItem* viewItem;
	FolderWidgetItem* nextViewItem;

	folderParts=QStringList::split('/',fileItem->url().path());
	folderIter=folderParts.begin();
	endFolderIter=folderParts.end();
	viewItem=static_cast<FolderWidgetItem*>(firstChild());

// Finds the deepest existing view item
	for(;folderIter!=endFolderIter;++folderIter) {
		nextViewItem=viewItem->find(*folderIter);
		if (nextViewItem) {
			folder+=*folderIter + "/";
			viewItem=nextViewItem;
		} else {
			break;
		}
	}

// Already exist, strange...
	if (folderIter==endFolderIter) {
		kdWarning() << "  dir already exists. Should not happen\n";
		return;
	}

// Creates missing view item
	folder+=*folderIter + "/";
	viewItem=new FolderWidgetItem(this,viewItem,*folderIter,folder);
	KURL url;
	url.setPath(folder);
	mItemMap[fileItem]=viewItem;
	if ( url.cmp(mURL,true)) {
		viewItem->setOpen(true);
		blockSignals(true);
		setSelected(viewItem,true);
		blockSignals(false);
		ensureItemVisible(viewItem);
	} else if (url.isParentOf(mURL)) {
		viewItem->setOpen(true);
	} else {
		viewItem->initExpandable();
	}

}




FolderWidgetItem* FolderWidget::find(const KURL& url) const {
	QStringList folderParts;
	QStringList::Iterator folderIter,endFolderIter;
	FolderWidgetItem* viewItem;

	folderParts=QStringList::split('/',url.path());
	endFolderIter=folderParts.end();

	viewItem=static_cast<FolderWidgetItem*>(firstChild());
	for(folderIter=folderParts.begin();
		folderIter!=endFolderIter;
		++folderIter) {

		viewItem=viewItem->find(*folderIter);
		if (!viewItem) {
			return 0L;
		}
	}

	return viewItem;
}


//- Popup ----------------------------------------------------------
void FolderWidget::onRightButtonClicked(QListViewItem*,const QPoint& pos,int) {
	mPopupMenu->popup(pos);
}


void FolderWidget::makeDir() {
	KIO::Job* job;
	if (!currentItem()) return;

	bool ok;
	QString newDir=KLineEditDlg::getText(i18n("Enter the name of the new folder :"),"",&ok,this);
	if (!ok) return;
	
	KURL newURL(mURL);
	newURL.addPath(newDir);
	job=KIO::mkdir(newURL);
	
	connect(job,SIGNAL(result(KIO::Job*)),
		this,SLOT(onDirMade(KIO::Job*)));
}


void FolderWidget::onDirMade(KIO::Job* job) {
	if (job->error()) {
		job->showErrorDialog(this);
		return;
	}
	if (!currentItem()) return;
	currentItem()->setOpen(true);
}


void FolderWidget::renameDir() {
	KIO::Job* job;
	if (!currentItem()) return;
	
	bool ok;
	QString newDir=KLineEditDlg::getText(i18n("Rename this folder to :"),mURL.filename(),&ok,this);
	if (!ok) return;

	KURL newURL=mURL.upURL();
	
	newURL.addPath(newDir);
	job=KIO::rename(mURL,newURL,false);
	mURL=newURL;

	connect(job,SIGNAL(result(KIO::Job*)),
		this,SLOT(onDirRenamed(KIO::Job*)));
}


void FolderWidget::onDirRenamed(KIO::Job* job) {
	if (job->error()) {
		job->showErrorDialog(this);
		return;
	}
}


void FolderWidget::removeDir() {
	KIO::Job* job;
	if (!currentItem()) return;

	int response=KMessageBox::questionYesNo(this,
		i18n("Are you sure you want to delete the folder <b>%1</b> ?").arg(mURL.path() ));
	if (response==KMessageBox::No) return;

	job=KIO::del(mURL);
	connect(job,SIGNAL(result(KIO::Job*)),
		this,SLOT(onDirRemoved(KIO::Job*)));
}


void FolderWidget::onDirRemoved(KIO::Job* job) {
	if (job->error()) {
		job->showErrorDialog(this);
	}
}


void FolderWidget::showPropertiesDialog() {
	(void)new KPropertiesDialog(mURL);
}

