/*
Gwenview - A simple image viewer for KDE
Copyright (C) 2000-2001 Aurlien Gteau

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

// KDE includes
#include <kaccel.h>
#include <kaction.h>
#include <kapp.h>
#include <kcmdlineargs.h>
#include <kconfig.h>
#include <kdebug.h>
#include <kfiledetailview.h>
#include <kfiledialog.h>
#include <kglobal.h>
#include <khelpmenu.h>
#include <kkeydialog.h>
#include <klocale.h>
#include <kmenubar.h>
#include <kmessagebox.h>
#include <kprogress.h>
#include <kstatusbar.h>
#include <kstdaccel.h>
#include <kstdaction.h>

// Our includes
#include "configdialog.h"
#include "filethumbnailview.h"
#include "filewidget.h"
#include "folderwidget.h"
#include "gvpixmap.h"
#include "pixmapwidget.h"
#include "statusbarprogress.h"

#include "mainwindow.moc"


enum { SB_FOLDER, SB_FILE};

const char* CONFIG_DOCK_GROUP="dock";
const char* CONFIG_MAINWINDOW_GROUP="main window";
const char* CONFIG_FILEWIDGET_GROUP="file widget";
const char* CONFIG_PIXMAPWIDGET_GROUP="pixmap widget";
const char* CONFIG_START_WITH_THUMBNAILS="start with thumbnails";
const char* CONFIG_ZOOM_STEP="zoom step";


MainWindow::MainWindow()
: KDockMainWindow(), mProgress(0L), mFullScreen(false), mZoomStep(0.5)
{
// Load config
	KConfig* config=KGlobal::config();
	config->setGroup(CONFIG_MAINWINDOW_GROUP);
	mStartWithThumbnails=config->readBoolEntry(CONFIG_START_WITH_THUMBNAILS,false);
	mZoomStep=config->readDoubleNumEntry(CONFIG_ZOOM_STEP,0.5);

// Backend
	mGVPixmap=new GVPixmap(this);

// GUI
	createWidgets();
	createActions();
	createAccels();
	createMenu();
	createToolBar();

// Connections
	connect(mFolderWidget,SIGNAL(folderChanged(QString)),
		this,SLOT(setPath(QString)) );

	connect(mPixmapWidget,SIGNAL(selectPrevious()),
		mFileWidget,SLOT(selectPrevious()) );
	connect(mPixmapWidget,SIGNAL(selectNext()),
		mFileWidget,SLOT(selectNext()) );

	connect(mFileWidget->fileThumbnailView(),SIGNAL(updateStarted(int)),
		this,SLOT(thumbnailUpdateStarted(int)) );
	connect(mFileWidget->fileThumbnailView(),SIGNAL(updateEnded()),
		this,SLOT(thumbnailUpdateEnded()) );
	connect(mFileWidget->fileThumbnailView(),SIGNAL(updatedOneThumbnail()),
		this,SLOT(thumbnailUpdateProcessedOne()) );

	connect(mFileWidget,SIGNAL(filenameChanged(QString)),
		this,SLOT(setFilename(QString)) );
	connect(mFileWidget,SIGNAL(completed()),
		this,SLOT(updateStatusBar()) );
	connect(mFileWidget,SIGNAL(canceled()),
		this,SLOT(updateStatusBar()) );

	connect(mGVPixmap,SIGNAL(loading()),
		this,SLOT(pixmapLoading()) );
	connect(mGVPixmap,SIGNAL(loaded()),
		this,SLOT(pixmapLoaded()) );

// Command line
	KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

	QString fullPath;
	if (args->count()==0) {
		fullPath=QDir::currentDirPath();
	} else {
		fullPath=QFile::decodeName(args->arg(0));
	}

// Apply config
	updateZoomGUI();

// Go to requested file
	setPath(fullPath);
}


MainWindow::~MainWindow() {
	KConfig* config=KGlobal::config();

	config->setGroup(CONFIG_MAINWINDOW_GROUP);
	config->writeEntry(CONFIG_START_WITH_THUMBNAILS,!mNoThumbnails->isChecked());
	config->writeEntry(CONFIG_ZOOM_STEP,mZoomStep);
	mPixmapWidget->writeConfig(config,CONFIG_PIXMAPWIDGET_GROUP);
	mFileWidget->writeConfig(config,CONFIG_FILEWIDGET_GROUP);
	writeDockConfig(config,CONFIG_DOCK_GROUP);
	mAccel->writeSettings();
}


//-Public slots----------------------------------------------------------
void MainWindow::setPath(QString fullPath) {
	QFileInfo fileInfo(fullPath);

// Init mFolder and mFilename
	if (fileInfo.isFile()) {
		mFolder=fileInfo.dirPath(true);
		mFilename=fileInfo.fileName();
		mGVPixmap->load(fullPath);
	} else {
		mFolder=fileInfo.absFilePath();
		mFilename="";
	}

// Update widgets
	mFileWidget->setFolderAndFile(mFolder,mFilename);
	mFolderWidget->setFolder(mFolder);
	if (mFilename!="") {
		mGVPixmap->load(fullPath);
	} else {
		mGVPixmap->reset();
	}
}


void MainWindow::setFilename(QString filename) {
	if (mFilename==filename) return;

	mFilename=filename;

	if (mFilename!="") {
		mFileWidget->selectFilename(mFilename);
		mGVPixmap->load(mFolder + '/' + mFilename);
	} else {
		mGVPixmap->reset();
	}
	updateFileStatusBar();
}


//-Private slots---------------------------------------------------------
void MainWindow::pixmapLoading() {
	kapp->setOverrideCursor(WaitCursor);
}


void MainWindow::pixmapLoaded() {
	kapp->restoreOverrideCursor();
	if ( (!mFilename.isEmpty()) && mGVPixmap->isNull()) {
		KMessageBox::sorry(this,i18n("Could not load <b>%1</b>").arg(mFilename));
	}
}


void MainWindow::setThumbnailSize() {
	if (mNoThumbnails->isChecked()) {
		mFileWidget->setMode(FileWidget::FileList);
		return;
	} else {
		if (mSmallThumbnails->isChecked()) {
			mFileWidget->fileThumbnailView()->setThumbnailSize(ThumbnailSize::Small);
		} else if (mMedThumbnails->isChecked()) {
			mFileWidget->fileThumbnailView()->setThumbnailSize(ThumbnailSize::Med);
		} else {
			mFileWidget->fileThumbnailView()->setThumbnailSize(ThumbnailSize::Large);
		}
		mFileWidget->setMode(FileWidget::Thumbnail);
	}
}


void MainWindow::toggleFullScreen() {
	KConfig* config=KGlobal::config();

	if (!mFullScreen) {
		mFullScreen=true;
		menuBar()->hide();
		toolBar()->hide();
		statusBar()->hide();
		writeDockConfig(config,CONFIG_DOCK_GROUP);
		makeDockInvisible(mFileDock);
		makeDockInvisible(mFolderDock);
		showFullScreen();
		setFocus(); // If this is not done, actions can't be called from keyboard !

		mPixmapWidget->setFullScreen(true);
	} else {
		mFullScreen=false;
		readDockConfig(config,CONFIG_DOCK_GROUP);
		statusBar()->show();
		toolBar()->show();
		menuBar()->show();
		showNormal();

		mPixmapWidget->setFullScreen(false);
	}
}


void MainWindow::showConfigDialog() {
	ConfigDialog dialog(this,this);
	dialog.exec();
}


void MainWindow::showKeyDialog() {
	KKeyDialog::configureKeys(mAccel);
}


void MainWindow::zoomIn() {
	mPixmapWidget->setScale(mPixmapWidget->scale() + mZoomStep);
	updateZoomGUI();
	updateFileStatusBar();
}


void MainWindow::zoomOut() {
	mPixmapWidget->setScale(mPixmapWidget->scale() - mZoomStep);
	updateZoomGUI();
	updateFileStatusBar();
}


void MainWindow::resetZoom() {
	mPixmapWidget->setScale(1.0);
	updateZoomGUI();
	updateFileStatusBar();
}


void MainWindow::escapePressed() {
	if (mFullScreen) {
		mToggleFullScreen->activate();
	}
}


void MainWindow::openFile() {
	QString path=KFileDialog::getOpenFileName();
	if (path.isNull()) return;
	setPath(path);
}


void MainWindow::thumbnailUpdateStarted(int count) {
	mProgress=new StatusBarProgress(statusBar(),i18n("Generating thumbnails..."),count);
	mProgress->progress()->setFormat("%v/%m");
	mProgress->show();
	mStop->setEnabled(true);
}


void MainWindow::thumbnailUpdateEnded() {
	mStop->setEnabled(false);
	if (mProgress) {
		mProgress->hide();
		delete mProgress;
		mProgress=0L;
	}
}


void MainWindow::thumbnailUpdateProcessedOne() {
	mProgress->progress()->advance(1);
}



//-Accessors-------------------------------------------------------------
void MainWindow::setZoomStep(double value) {
	mZoomStep=value;
	updateZoomGUI();
}


double MainWindow::zoomStep() const {
	return mZoomStep;
}


//-GUI-------------------------------------------------------------------
void MainWindow::updateZoomGUI() {
	mZoomOut->setEnabled(mPixmapWidget->scale()-mZoomStep>0);
}


void MainWindow::updateStatusBar() {
	QString txt;
	uint count=mFileWidget->fileCount();
	if (count>1) {
		txt=i18n("%1 - %2 images");
	} else {
		txt=i18n("%1 - %2 image");
	}
	txt=txt.arg(mFolder).arg(count);

	statusBar()->changeItem( txt, SB_FOLDER );
	updateFileStatusBar();
}


void MainWindow::updateFileStatusBar() {
	QString txt;
	if (!mFilename.isEmpty()) {
		txt=QString("%1 %2x%3 @ %4%")
			.arg(mFilename).arg(mGVPixmap->width()).arg(mGVPixmap->height())
			.arg(int(mPixmapWidget->scale()*100) );
	} else {
		txt="";
	}
	statusBar()->changeItem( txt, SB_FILE );
}


void MainWindow::createWidgets() {
	KConfig* config=KGlobal::config();

// Status bar
	statusBar()->insertItem("",SB_FOLDER);
	statusBar()->setItemAlignment(SB_FOLDER, AlignLeft|AlignVCenter);
	statusBar()->insertItem("",SB_FILE,1);
	statusBar()->setItemAlignment(SB_FILE, AlignLeft|AlignVCenter);

// Pixmap widgets
	mPixmapDock = createDockWidget("Image",SmallIcon("gwenview"),NULL,i18n("Image"));

	mPixmapWidget=new PixmapWidget(mPixmapDock,mGVPixmap);
	mPixmapDock->setWidget(mPixmapWidget);
	setView(mPixmapDock);
	setMainDockWidget(mPixmapDock);

// Folder widget
	mFolderDock = createDockWidget("Folders",SmallIcon("folder_open"),NULL,i18n("Folders"));
	mFolderWidget=new FolderWidget(mFolderDock);
	mFolderDock->setWidget(mFolderWidget);

// File widget
	mFileDock = createDockWidget("Files",SmallIcon("image"),NULL,i18n("Files"));
	mFileWidget=new FileWidget(
		this,
		mStartWithThumbnails?FileWidget::Thumbnail:FileWidget::FileList);
	mFileDock->setWidget(mFileWidget);

// Default dock config
	setGeometry(20,20,600,400);
	mFolderDock->manualDock( mPixmapDock,KDockWidget::DockLeft,30);
	mFileDock->manualDock( mPixmapDock,KDockWidget::DockTop,30);

// Load config
	readDockConfig(config,CONFIG_DOCK_GROUP);
	mFileWidget->readConfig(config,CONFIG_FILEWIDGET_GROUP);
	mPixmapWidget->readConfig(config,CONFIG_PIXMAPWIDGET_GROUP);
}


void MainWindow::createActions() {
	mOpenFile=KStdAction::open(this, SLOT(openFile()),actionCollection() );

	mRenameFile=new KAction(i18n("&Rename..."),Key_F2,mFileWidget,SLOT(renameFile()),actionCollection(),"file_new");

	mCopyFile=new KAction(i18n("&Copy to..."),Key_F5,mFileWidget,SLOT(copyFile()),actionCollection(),"file_copy");

	mMoveFile=new KAction(i18n("&Move to..."),Key_F6,mFileWidget,SLOT(moveFile()),actionCollection(),"file_move");

	mDeleteFile=new KAction(i18n("&Delete"),"editdelete",Key_Delete,mFileWidget,SLOT(deleteFile()),actionCollection(),"file_delete");

	mNoThumbnails=new KRadioAction(i18n("Details"),"view_detailed",0,this,SLOT(setThumbnailSize()),actionCollection(),"view_details");
	mNoThumbnails->setExclusiveGroup("thumbnails");
	mSmallThumbnails=new KRadioAction(i18n("Small thumbnails"),"smallthumbnails",0,this,SLOT(setThumbnailSize()),actionCollection(),"view_small_thumbnails");
	mSmallThumbnails->setExclusiveGroup("thumbnails");
	mMedThumbnails=new KRadioAction(i18n("Medium thumbnails"),"medthumbnails",0,this,SLOT(setThumbnailSize()),actionCollection(),"view_med_thumbnails");
	mMedThumbnails->setExclusiveGroup("thumbnails");
	mLargeThumbnails=new KRadioAction(i18n("Large thumbnails"),"largethumbnails",0,this,SLOT(setThumbnailSize()),actionCollection(),"view_large_thumbnails");
	mLargeThumbnails->setExclusiveGroup("thumbnails");

	if (mStartWithThumbnails) {
		switch (mFileWidget->fileThumbnailView()->thumbnailSize()) {
		case ThumbnailSize::Small:
			mSmallThumbnails->setChecked(true);
			break;
		case ThumbnailSize::Med:
			mMedThumbnails->setChecked(true);
			break;
		case ThumbnailSize::Large:
			mLargeThumbnails->setChecked(true);
			break;
		}
	} else {
		mNoThumbnails->setChecked(true);
	}

	mToggleFullScreen=new KAction(i18n("Full screen"),"window_fullscreen",CTRL + Key_F,this,SLOT(toggleFullScreen()),actionCollection(),"view_fullscreen");

	mSelectFirst=new KAction(i18n("&First"),"start",Key_Home,
		mFileWidget,SLOT(selectFirst()),actionCollection(),"view_first");

	mSelectLast=new KAction(i18n("&Last"),"finish",Key_End,
		mFileWidget,SLOT(selectLast()),actionCollection(),"view_last");

	mSelectPrevious=KStdAction::back(mFileWidget, SLOT(selectPrevious()),actionCollection() );
	mSelectPrevious->setAccel(Key_Backspace);

	mSelectNext=KStdAction::forward(mFileWidget, SLOT(selectNext()),actionCollection() );
	mSelectNext->setAccel(Key_Space);


	mShowConfigDialog=new KAction(i18n("Configure Gwenview"),"configure",0,this,SLOT(showConfigDialog()),actionCollection());

	mShowKeyDialog=KStdAction::keyBindings(this,SLOT(showKeyDialog()),actionCollection());

	mZoomIn=KStdAction::zoomIn(this,SLOT(zoomIn()),actionCollection());

	mZoomOut=KStdAction::zoomOut(this,SLOT(zoomOut()),actionCollection());

	mResetZoom=KStdAction::actualSize(this,SLOT(resetZoom()),actionCollection());
	mResetZoom->setIcon("actualsize");

	mStop=new KAction(i18n("Stop"),"stop",Key_Escape,mFileWidget,SLOT(cancel()),actionCollection());
	mStop->setEnabled(false);
}


void MainWindow::createAccels() {
// Associate actions with accelerator
	mAccel=new KAccel(this);
	int count=actionCollection()->count();

	for (int pos=0;pos<count;++pos) {
		KAction *action = actionCollection()->action(pos);
		if ( action->accel() ) {
			action->plugAccel(mAccel);
		}
	}

// Read user accelerator
	mAccel->readSettings();

// Special accelerator for full screen exit.
// Using a special KAccel ensures it can't be configured.
// This is necessary since the 'configurable' argument in KAccel::insertItem
// seems to be broken (KDE 2.1.1)
	KAccel* accel=new KAccel(this);
	accel->insertItem("","fullScreenClose",Key_Escape,this);
	accel->connectItem("fullScreenClose",this,SLOT(escapePressed()) );
}


void MainWindow::createMenu() {
	QPopupMenu* fileMenu = new QPopupMenu;

	mOpenFile->plug(fileMenu);
	fileMenu->insertSeparator();
	mRenameFile->plug(fileMenu);
	mCopyFile->plug(fileMenu);
	mMoveFile->plug(fileMenu);
	mDeleteFile->plug(fileMenu);
	fileMenu->insertSeparator();
	KStdAction::quit( kapp, SLOT (closeAllWindows()) )->plug(fileMenu);
	menuBar()->insertItem(i18n("&File"), fileMenu);

	QPopupMenu* viewMenu = new QPopupMenu;
	mStop->plug(viewMenu);
	viewMenu->insertSeparator();
	mNoThumbnails->plug(viewMenu);
	mSmallThumbnails->plug(viewMenu);
	mMedThumbnails->plug(viewMenu);
	mLargeThumbnails->plug(viewMenu);
	viewMenu->insertSeparator();

	mToggleFullScreen->plug(viewMenu);
	mZoomIn->plug(viewMenu);
	mZoomOut->plug(viewMenu);
	mResetZoom->plug(viewMenu);
	menuBar()->insertItem(i18n("&View"), viewMenu);

	QPopupMenu* goMenu = new QPopupMenu;
	mSelectFirst->plug(goMenu);
	mSelectPrevious->plug(goMenu);
	mSelectNext->plug(goMenu);
	mSelectLast->plug(goMenu);
	menuBar()->insertItem(i18n("&Go"), goMenu);

	QPopupMenu* settingsMenu = new QPopupMenu;
	mShowConfigDialog->plug(settingsMenu);
	mShowKeyDialog->plug(settingsMenu);
	menuBar()->insertItem(i18n("&Settings"),settingsMenu);

	menuBar()->insertItem(i18n("&Windows"), dockHideShowMenu());

	menuBar()->insertItem(i18n("&Help"), helpMenu());

	menuBar()->show();
}


void MainWindow::createToolBar() {
	mSelectFirst->plug(toolBar());
	mSelectPrevious->plug(toolBar());
	mSelectNext->plug(toolBar());
	mSelectLast->plug(toolBar());
	toolBar()->insertLineSeparator();

	mStop->plug(toolBar());
	toolBar()->insertLineSeparator();

	mNoThumbnails->plug(toolBar());
	mSmallThumbnails->plug(toolBar());
	mMedThumbnails->plug(toolBar());
	mLargeThumbnails->plug(toolBar());
	toolBar()->insertLineSeparator();

	mToggleFullScreen->plug(toolBar());
	mZoomIn->plug(toolBar());
	mZoomOut->plug(toolBar());
	mResetZoom->plug(toolBar());
	toolBar()->show();
}

